type vect2D == float * float
;;
type vect3D == float * float * float
;;
type rotation == vect3D * float
;;
type color == float * float * float
;;
type time == float
;;
type frame = 
  { height : float * float
  ; width : float * float
  ; depth : float * float
  }
;;
type clip = 
  { clip_loop : bool
  ; clip_start_time : time
  ; clip_stop_time : time
  ; clip_urls : string list
  }
;;
type fontStyle = 
  { family : string
  ; horizontal : bool
  ; justify : string list
  ; language : string
  ; left_to_right : bool
  ; size : float
  ; spacing : float
  ; style : string
  ; top_to_bottom : bool
  }
;;
type image = 
  { image_width : int
  ; image_height : int
  ; image_components : int
  ; image_pixels : int list
  }
;;
type textureCoordinate == vect2D list
;;
type movieTexture = 
  { movieTexture_speed : float
  ; movieTexture_clip : clip
  }
;;
type textureType = 
    ImageTexture of string list
  | MovieTexture of movieTexture
  | PixelTexture of image
;;
type textureItem = 
  { texture_repeatS : bool
  ; texture_repeatT : bool
  ; texture_type : textureType
  ; texture_center : vect2D
  ; texture_rotation : float
  ; texture_scale : vect2D
  ; texture_translation : vect2D
  }
;;
type texture = 
    NoTexture
  | Texture of textureItem
;;
type materialItem = 
  { ambient_intensity : float
  ; diffuse_color : color
  ; emissive_color : color
  ; shininess : float
  ; specular_color : color
  ; transparency : float
  }
;;
type material = 
    NoMaterial
  | Material of materialItem
;;
type point = 
  { point_color : color
  ; point_coordinate : vect3D
  }
;;
type pointSet == point list
;;
type line = 
  { line_point1 : vect3D
  ; line_point2 : vect3D
  ; line_color : color
  }
;;
type lineSet == line list
;;
type unlitGeometric = 
    LineSet of lineSet
  | PointSet of pointSet
;;
type box = 
  { box_size : vect3D
  }
;;
type cone = 
  { cone_bottom_radius : float
  ; cone_height : float
  ; cone_side : bool
  ; cone_bottom : bool
  }
;;
type cylinder = 
  { cylinder_bottom : bool
  ; cylinder_height : float
  ; cylinder_radius : float
  ; cylinder_side : bool
  ; cylinder_top : bool
  }
;;
type elevationGrid = 
  { grid_color : color list
  ; grid_texture_coordinate : textureCoordinate
  ; grid_height : float list
  ; grid_ccw : bool
  ; grid_color_per_vertex : bool
  ; grid_solid : bool
  ; grid_x_dimension : int
  ; grid_x_spacing : float
  ; grid_z_dimension : int
  ; grid_z_spacing : float
  }
;;
type extrusion = 
  { extrusion_begin_cap : bool
  ; extrusion_ccw : bool
  ; extrusion_convex : bool
  ; extrusion_cross_section : vect2D list
  ; extrusion_end_cap : bool
  ; extrusion_orientation : rotation list
  ; extrusion_scale : vect2D list
  ; extrusion_solid : bool
  ; extrusion_spine : vect3D list
  }
;;
type sphere = 
  { sphere_radius : float
  }
;;
type text = 
  { content : string list
  ; font_style : fontStyle
  ; length : float list
  ; max_extent : float
  }
;;
type face == vect3D list
;;
type faceSet = 
  { faceSet_faces : face list
  ; faceSet_texture_coordinate : textureCoordinate
  ; faceSet_ccw : bool
  ; faceSet_convex : bool
  ; faceSet_solid : bool
  ; faceSet_texture_coordinate_index : int list
  }
;;
type shape = 
    Box of box
  | Cone of cone
  | Cylinder of cylinder
  | ElevationGrid of elevationGrid
  | Extrusion of extrusion
  | FaceSet of faceSet
  | Sphere of sphere
  | Text of text
;;
type directionalLight = 
  { directionalLight_direction : vect3D
  }
;;
type pointLight = 
  { pointLight_attenuation : vect3D
  ; pointLight_radius : float
  }
;;
type spotLight = 
  { spotLight_attenuation : vect3D
  ; spotLight_beam_width : float
  ; spotLight_cut_off_angle : float
  ; spotLight_direction : vect3D
  ; spotLight_radius : float
  }
;;
type lightType = 
    DirectionalLight of directionalLight
  | PointLight of pointLight
  | SpotLight of spotLight
;;
type light = 
  { light_ambient_intensity : float
  ; light_color : color
  ; light_intensity : float
  ; light_type : lightType
  }
;;
type geometric = 
  { shape : shape
  ; material : material
  ; texture : texture
  }
;;
type viewPoint = 
  { viewPoint_field_of_view : float
  ; viewPoint_jump : bool
  ; viewPoint_description : string
  ; navigation_avatar_size : float list
  ; navigation_headlight : bool
  ; navigation_speed : float
  ; navigation_type : string list
  ; navigation_visibility_limit : float
  }
;;
type sound = 
  { sound_direction : vect3D
  ; sound_intensity : float
  ; sound_max_back : float
  ; sound_max_front : float
  ; sound_min_back : float
  ; sound_min_front : float
  ; sound_priority : float
  ; sound_spatialize : bool
  ; sound_description : string
  ; sound_pitch : float
  ; sound_clip : clip
  }
;;
type background = 
  { ground_angle : float list
  ; ground_color : color list
  ; back_url : string list
  ; bottom_url : string list
  ; front_url : string list
  ; left_url : string list
  ; right_url : string list
  ; top_url : string list
  ; sky_angle : float list
  ; sky_color : color list
  }
;;
type fog = 
  { fog_color : color
  ; fog_type : string
  ; fog_visibility_range : float
  }
;;
type vrml_special = 
    Background of background
  | Fog of fog
;;
type anchorItem = 
  { anchor_description : string
  ; anchor_parameters : string list
  ; anchor_urls : string list
  }
;;
type anchor = 
    NoAnchor
  | Anchor of anchorItem
;;
type billboardItem = 
  { axis_of_rotation : vect3D
  }
;;
type billboard = 
    NoBillboard
  | Billboard of billboardItem
;;
type collisionBox = 
  { collide : bool
  ; bbox_center : vect3D
  ; bbox_size : vect3D
  }
;;
type collision = 
    NoCollision
  | Collision of collisionBox
;;
type proximityBox = 
  { proximity_center : vect3D
  ; proximity_size : vect3D
  }
;;
type vrml_object = 
  { objects : vrml_positionable list
  ; children : vrml_object list
  ; position : vect3D
  ; rotation : rotation
  ; scale : vect3D
  ; frame : frame
  ; unique : bool
  ; anchor : anchor
  ; billboard : billboard
  ; collision : collision
  }
 and vrml_positionable = 
    Geometric of geometric
  | UnlitGeometric of unlitGeometric
  | CollisionBox of collisionBox
  | Light of light
  | LOD of lod
  | Sound of sound
  | ViewPoint of viewPoint
 and lod = 
  { lod_levels : vrml_object list
  ; lod_ranges : float list
  }
;;
type changing = 
    Translation of float list * vect3D list
  | Rotation of float list * rotation list
  | Scale of float list * vect3D list
  | ChangeColor
;;
type trigger = 
    Instant of time
  | Click of vrml_object
  | Enter of proximityBox * vrml_object
  | Exit of proximityBox * vrml_object
;;
type animation = 
  { target : vrml_object
  ; start_trigger : trigger list
  ; stop_trigger : trigger list
  ; loop : bool
  ; duration : float
  ; memory : bool
  ; movie : changing list
  }
;;
type vrml_world = 
  { info : string list
  ; title : string
  ; tree : vrml_object
  ; specials : vrml_special list
  ; animations : animation list
  }
;;

(* compatibility.ml *)
value space_char : char;;
value lf_char : char;;
value char_0 : char;;
value comma_char : char;;
value open_par_char : char;;
value close_par_char : char;;
value ascii_F : int;;
value ascii_A : int;;
value ascii_f : int;;
value ascii_a : int;;
value ascii_9 : int;;
value ascii_0 : int;;
value append : 'a list -> 'a list -> 'a list;;
value lnot : int -> int;;
value format_float : string -> float -> string;;
value combine : 'a list -> 'b list -> ('a * 'b) list;;

(* default.ml *)
value default_frame : frame;;
value default_clip : clip;;
value default_fontStyle : fontStyle;;
value default_image : image;;
value default_vrml_object : vrml_object;;
value default_anchor : anchorItem;;
value default_basic_anchor : anchor;;
value default_billboard : billboardItem;;
value default_basic_billboard : billboard;;
value default_collisionBox : collisionBox;;
value default_basic_collision : collision;;
value default_material : materialItem;;
value default_basic_material : material;;
value default_textureCoordinate : 'a list;;
value default_basic_imageTexture : textureType;;
value default_movieTexture : movieTexture;;
value default_basic_movieTexture : textureType;;
value default_basic_pixelTexture : textureType;;
value default_basic_texture : textureType -> texture;;
value default_texture : textureType -> textureItem;;
value default_textureItem : textureItem;;
value default_basic_geometric : shape -> vrml_positionable;;
value default_point : point;;
value default_pointSet : 'a list;;
value default_basic_pointSet : unlitGeometric;;
value default_line : line;;
value default_lineSet : 'a list;;
value default_basic_lineSet : unlitGeometric;;
value default_box : box;;
value default_basic_box : shape;;
value default_cone : cone;;
value default_basic_cone : shape;;
value default_cylinder : cylinder;;
value default_basic_cylinder : shape;;
value default_elevationGrid : elevationGrid;;
value default_basic_elevationGrid : shape;;
value default_extrusion : extrusion;;
value default_basic_extrusion : shape;;
value default_sphere : sphere;;
value default_basic_sphere : shape;;
value default_text : text;;
value default_basic_text : shape;;
value default_face : 'a list;;
value default_faceSet : faceSet;;
value default_directionalLight : directionalLight;;
value default_basic_directionalLight : lightType;;
value default_pointLight : pointLight;;
value default_basic_pointLight : lightType;;
value default_spotLight : spotLight;;
value default_basic_spotLight : lightType;;
value default_basic_light : lightType -> vrml_positionable;;
value default_light : light;;
value default_sound : sound;;
value default_basic_sound : vrml_positionable;;
value default_lod : lod;;
value default_basic_lod : vrml_positionable;;
value default_viewPoint : viewPoint;;
value default_basic_viewPoint : vrml_positionable;;
value default_background : background;;
value default_basic_background : vrml_special;;
value default_fog : fog;;
value default_basic_fog : vrml_special;;
value default_vrml_world : vrml_world;;

(* constants.ml *)
value pi : float;;

(* exceptions.ml *)
value test_color : color -> color;;
value test_time : time -> time;;
value test_speed : float -> float;;
value test_size : vect3D -> vect3D;;
value test_proximity_size : vect3D -> vect3D;;
value test_radius : float -> float;;
value test_height : float -> float;;
value test_length : float -> float;;
value test_max_extent : float -> float;;
value test_elevationGrid_dimension : int -> int;;
value test_elevationGrid_spacing : float -> float;;
value test_extrusion_scale : vect2D list -> vect2D list;;
value test_material_ambient_intensity : float -> float;;
value test_material_shininess : float -> float;;
value test_material_transparency : float -> float;;
value test_attenuation : vect3D -> vect3D;;
value test_cut_off_angle : float -> float;;
value test_beam_width : float -> float;;
value test_light_intensity : float -> float;;
value test_light_ambient_intensity : float -> float;;
value test_field_of_view : float -> float;;
value test_avatar_size : float -> float;;
value test_visibility_limit : float -> float;;
value test_sound_min_or_max : float -> float;;
value test_sound_intensity : float -> float;;
value test_sound_priority : float -> float;;
value test_pitch : float -> float;;
value test_background_angle : float -> float;;

(* basic_types.ml *)
value new_vect2D : float -> float -> vect2D;;
value new_vect3D : float -> float -> float -> vect3D;;
value new_rotation : float -> float -> float -> float -> rotation;;
value new_color : float -> float -> float -> color;;
value new_time : time -> time;;
value new_frame : float -> float -> float -> float -> float -> float -> frame;;
value new_point : color -> float -> float -> float -> point;;
value setf_point_color : point -> color -> point;;
value setf_point_coordinate : point -> float -> float -> float -> point;;
value new_line : color -> vect3D -> vect3D -> line;;
value setf_line_point1 : line -> vect3D -> line;;
value setf_line_point2 : line -> vect3D -> line;;
value setf_line_color : line -> color -> line;;
value new_face : face -> face;;
value new_complete_clip : bool -> time -> time -> string list -> clip;;
value new_clip : time -> time -> string -> clip;;
value setf_clip_loop : clip -> bool -> clip;;
value setf_clip_start_time : clip -> time -> clip;;
value setf_clip_stop_time : clip -> time -> clip;;
value setf_clip_urls : clip -> string list -> clip;;
value new_image : int -> int -> int -> int list -> image;;
value setf_image_width : image -> int -> image;;
value setf_image_height : image -> int -> image;;
value setf_image_components : image -> int -> image;;
value setf_image_pixels : image -> int list -> image;;
value new_basic_collisionBox : bool -> vect3D -> vect3D -> collisionBox;;

(* font.ml *)
value new_complete_font : string -> bool -> string list -> string -> bool -> float -> float -> string -> bool -> fontStyle;;
value new_font : string -> float -> string -> fontStyle;;
value setf_family : fontStyle -> string -> fontStyle;;
value setf_horizontal : fontStyle -> bool -> fontStyle;;
value setf_justify : fontStyle -> string list -> fontStyle;;
value setf_language : fontStyle -> string -> fontStyle;;
value setf_left_to_right : fontStyle -> bool -> fontStyle;;
value setf_size : fontStyle -> float -> fontStyle;;
value setf_spacing : fontStyle -> float -> fontStyle;;
value setf_style : fontStyle -> string -> fontStyle;;
value setf_top_to_bottom : fontStyle -> bool -> fontStyle;;
value serif : string;;
value sans_serif : string;;
value typewriter : string;;
value plain : string;;
value bold : string;;
value italic : string;;
value bold_italic : string;;
value set_font_family : fontStyle -> string -> fontStyle;;
value set_font_size : fontStyle -> float -> fontStyle;;
value set_font_style : fontStyle -> string -> fontStyle;;

(* fonts.ml *)
value plain_serif : fontStyle;;
value plain_sans_serif : fontStyle;;
value plain_typewriter : fontStyle;;
value bold_serif : fontStyle;;
value bold_sans_serif : fontStyle;;
value bold_typewriter : fontStyle;;
value italic_serif : fontStyle;;
value italic_sans_serif : fontStyle;;
value italic_typewriter : fontStyle;;
value bold_italic_serif : fontStyle;;
value bold_italic_sans_serif : fontStyle;;
value bold_italic_typewriter : fontStyle;;
value white : color;;
value black : color;;
value green : color;;
value red : color;;
value blue : color;;
value yellow : color;;
value magenta : color;;
value cyan : color;;
value dark_gray : color;;
value dark_blue : color;;
value dark_cyan : color;;
value dark_magenta : color;;
value dark_red : color;;
value light_green : color;;

(* vrml_objects_pre.ml *)
value new_referential : vrml_object;;
value new_vrml_object : vrml_positionable list -> vrml_object list -> vect3D -> rotation -> vect3D -> frame -> bool -> anchor -> billboard -> collision -> vrml_object;;
value setf_objects : vrml_object -> vrml_positionable list -> vrml_object;;
value set_objects : vrml_object -> vrml_positionable list -> vrml_object;;
value setf_children : vrml_object -> vrml_object list -> vrml_object;;
value set_children : vrml_object -> vrml_object list -> vrml_object;;
value setf_position : vrml_object -> vect3D -> vrml_object;;
value setf_rotation : vrml_object -> rotation -> vrml_object;;
value setf_scale : vrml_object -> vect3D -> vrml_object;;
value setf_frame : vrml_object -> frame -> vrml_object;;
value set_frame : vrml_object -> frame -> vrml_object;;
value setf_unique : vrml_object -> bool -> vrml_object;;
value set_unique : vrml_object -> bool -> vrml_object;;
value setf_anchor : vrml_object -> anchor -> vrml_object;;
value setf_billboard : vrml_object -> billboard -> vrml_object;;
value setf_collision : vrml_object -> collision -> vrml_object;;

(* calc_3D.ml *)
value rescale_3Dvect : float * float * float -> float * float * float -> float * float * float;;
value retranslate_3Dvect : float * float * float -> float * float * float -> float * float * float;;
value rotat_point : (float * float * float) * float -> float * float * float -> float * float * float;;
value frame_to_cube : frame -> (float * float * float) list;;
value translate_frame : frame -> float * float * float -> frame;;
value extract : ('a -> 'b) -> 'a list -> 'b * 'b;;
value give_x : 'a * 'b * 'c -> 'a;;
value give_y : 'a * 'b * 'c -> 'b;;
value give_z : 'a * 'b * 'c -> 'c;;
(* value identity : 'a -> 'a;; *)
value frame_to_frame : frame -> (float * float * float) * float -> float * float * float -> frame;;
value build_frame : frame -> (float * float * float) * float -> float * float * float -> float * float * float -> frame -> frame;;
value resize_frame_with_child : vrml_object -> vrml_object -> vrml_object;;
value nth : 'a list -> int -> 'a;;
value fill_frame : float list -> frame;;
value inv : float -> float;;
value member : 'a -> 'a list -> bool;;
value plane_list : 'a list list -> 'a list;;
value vertex_extract : 'a list -> 'a list -> 'a list;;
value extract_shape : 'a list -> ('a * 'a * 'a) list -> 'a list;;
value resize_frame : frame -> (float * float * float) list -> frame;;
value translate_point : float * float * float -> float * float * float -> float * float * float;;
value scale_point : float * float * float -> float * float * float -> float * float * float;;
value compute_point : (float * float * float) * float -> float * float * float -> float * float * float -> float * float * float -> float * float * float;;
value tridim_y : 'a -> 'b * 'c -> 'c * 'a * 'b;;
value rotat_list_point : ((float * float * float) * float) list -> float * float * float -> float * float * float;;
value extrusion_frame_make : extrusion -> frame;;
value geometric_frame_construct : geometric -> frame;;
value extract_from_list : ('a -> 'b list) -> 'a list -> 'b list;;
value unlitGeometric_vertex_construct : unlitGeometric -> vect3D list;;
value unlitGeometric_frame_construct : unlitGeometric -> frame;;
value vrml_positionable_frame_construct : vrml_positionable -> frame;;
value resize_frame_with_object : vrml_object -> vrml_positionable -> vrml_object;;
value get_resize_frame : vrml_object -> frame;;

(* vrml_objects_post.ml *)
value add_object : vrml_object -> vrml_positionable -> vrml_object;;
value add_objects : vrml_object -> vrml_positionable list -> vrml_object;;
value add_child : vrml_object -> vrml_object -> vrml_object;;
value add_children : vrml_object -> vrml_object list -> vrml_object;;
value set_position : vrml_object -> float -> float -> float -> vrml_object;;
value set_translation : vrml_object -> float -> float -> float -> vrml_object;;
value set_rotation : vrml_object -> float -> float -> float -> float -> vrml_object;;
value set_scale : vrml_object -> float -> float -> float -> vrml_object;;
value translation : vrml_object -> float -> float -> float -> vrml_object;;
value rotation : vrml_object -> float -> vrml_object;;
value add_rotation : vrml_object -> float -> float -> float -> float -> vrml_object;;
value resize : vrml_object -> float -> float -> float -> vrml_object;;
value group : vrml_object list -> vrml_object;;
value ungroup : vrml_object -> vrml_object list;;
value agregate_one : vrml_object -> vrml_object -> vrml_object;;
value objects_from_list : vrml_object list -> vrml_positionable list;;
value children_from_list : vrml_object list -> vrml_object list;;
value agregate : vrml_object list -> vrml_object -> vrml_object;;
value new_basic_anchor : string -> string list -> string list -> anchor;;
value set_complete_anchor : vrml_object -> string -> string list -> string list -> vrml_object;;
value set_urls : vrml_object -> string list -> vrml_object;;
value set_url : vrml_object -> string -> vrml_object;;
value add_url : vrml_object -> string -> vrml_object;;
value new_basic_billboard : float -> float -> float -> billboard;;
value set_billboard : vrml_object -> float -> float -> float -> vrml_object;;
value setf_collide : collisionBox -> bool -> collisionBox;;
value setf_bbox : collisionBox -> vect3D -> vect3D -> collisionBox;;
value setf_collision_collide : collision -> bool -> collision;;
value setf_collision_bbox : collision -> vect3D -> vect3D -> collision;;
value new_basic_collision : bool -> vect3D -> vect3D -> collision;;
value set_collidable : vrml_object -> vrml_object;;
value set_uncollidable : vrml_object -> vrml_object;;
value set_collision : vrml_object -> vect3D -> vect3D -> vrml_object;;

(* positionables.mlp *)
value new_basic_collision_box : vect3D -> vect3D -> vrml_positionable;;
value new_collision_box : vect3D -> vect3D -> vrml_object;;
value setf_collision_box_center : vrml_positionable -> vect3D -> vrml_positionable;;
value setf_collision_box_size : vrml_positionable -> vect3D -> vrml_positionable;;
value new_unlit_geometric_object : vrml_positionable -> vrml_object;;
value new_basic_pointSet : pointSet -> vrml_positionable;;
value new_pointSet : pointSet -> vrml_object;;
value add_basic_point : vrml_positionable -> point -> vrml_positionable;;
value new_basic_lineSet : lineSet -> vrml_positionable;;
value new_lineSet : lineSet -> vrml_object;;
value add_basic_line : vrml_positionable -> line -> vrml_positionable;;
value new_basic_viewPoint : float -> bool -> string -> float list -> bool -> float -> string list -> float -> vrml_positionable;;
value new_complete_viewPoint : float -> bool -> string -> float list -> bool -> float -> string list -> float -> vrml_object;;
value new_viewPoint : string -> vrml_object;;
value setf_viewPoint_field_of_view : vrml_positionable -> float -> vrml_positionable;;
value setf_viewPoint_jump : vrml_positionable -> bool -> vrml_positionable;;
value setf_viewPoint_description : vrml_positionable -> string -> vrml_positionable;;
value setf_navigation_avatar_sizes : vrml_positionable -> float list -> vrml_positionable;;
value setf_navigation_headlight : vrml_positionable -> bool -> vrml_positionable;;
value setf_navigation_speed : vrml_positionable -> float -> vrml_positionable;;
value setf_navigation_types : vrml_positionable -> string list -> vrml_positionable;;
value setf_navigation_visibility_limit : vrml_positionable -> float -> vrml_positionable;;
value new_basic_lod : vrml_object list -> float list -> vrml_positionable;;
value new_lod : vrml_object list -> float list -> vrml_object;;
value setf_lod_levels : vrml_positionable -> vrml_object list -> vrml_object;;
value setf_lod_ranges : vrml_positionable -> float list -> vrml_object;;
value add_lod_level_at_range : vrml_positionable -> vrml_object -> float -> vrml_object;;

(* geometrics.ml *)
value new_basic_geometric : shape -> material -> texture -> vrml_positionable;;
value setf_shape : vrml_positionable -> shape -> vrml_positionable;;
value setf_material : vrml_positionable -> material -> vrml_positionable;;
value setf_texture : vrml_positionable -> texture -> vrml_positionable;;
value new_geometric_object : shape -> vrml_object;;
value setf_shape_field : vrml_positionable -> (shape -> 'a -> shape) -> 'a -> vrml_positionable;;
value new_basic_box : float -> float -> float -> shape;;
value new_basic_cone : float -> float -> bool -> bool -> shape;;
value setf_basic_cone_radius : shape -> float -> shape;;
value setf_basic_cone_height : shape -> float -> shape;;
value setf_basic_cone_side : shape -> bool -> shape;;
value setf_basic_cone_bottom : shape -> bool -> shape;;
value setf_cone_radius : vrml_positionable -> float -> vrml_positionable;;
value setf_cone_height : vrml_positionable -> float -> vrml_positionable;;
value setf_cone_side : vrml_positionable -> bool -> vrml_positionable;;
value setf_cone_bottom : vrml_positionable -> bool -> vrml_positionable;;
value new_basic_cylinder : bool -> float -> float -> bool -> bool -> shape;;
value setf_basic_cylinder_bottom : shape -> bool -> shape;;
value setf_basic_cylinder_height : shape -> float -> shape;;
value setf_basic_cylinder_radius : shape -> float -> shape;;
value setf_basic_cylinder_side : shape -> bool -> shape;;
value setf_basic_cylinder_top : shape -> bool -> shape;;
value setf_cylinder_bottom : vrml_positionable -> bool -> vrml_positionable;;
value setf_cylinder_height : vrml_positionable -> float -> vrml_positionable;;
value setf_cylinder_radius : vrml_positionable -> float -> vrml_positionable;;
value setf_cylinder_side : vrml_positionable -> bool -> vrml_positionable;;
value setf_cylinder_top : vrml_positionable -> bool -> vrml_positionable;;
value new_basic_sphere : float -> shape;;
value new_basic_text : string list -> fontStyle -> float list -> float -> shape;;
value setf_basic_content : shape -> string list -> shape;;
value setf_basic_font_style : shape -> fontStyle -> shape;;
value setf_basic_length : shape -> float list -> shape;;
value setf_basic_max_extent : shape -> float -> shape;;
value setf_text_content : vrml_positionable -> string list -> vrml_positionable;;
value setf_text_font : vrml_positionable -> fontStyle -> vrml_positionable;;
value setf_text_length : vrml_positionable -> float list -> vrml_positionable;;
value setf_text_max_extent : vrml_positionable -> float -> vrml_positionable;;
value new_box : float -> float -> float -> vrml_object;;
value new_complete_cone : float -> float -> bool -> bool -> vrml_object;;
value new_cone : float -> float -> vrml_object;;
value new_complete_cylinder : float -> float -> bool -> bool -> bool -> vrml_object;;
value new_cylinder : float -> float -> vrml_object;;
value new_cube : float -> vrml_object;;
value new_sphere : float -> vrml_object;;
value new_complete_text : string list -> fontStyle -> float list -> float -> vrml_object;;
value new_text : string list -> vrml_object;;
value new_text_with_font : string list -> fontStyle -> vrml_object;;

(* geometric_properties.ml *)
value new_basic_material : float -> color -> color -> float -> color -> float -> material;;
value setf_basic_ambient_intensity : material -> float -> material;;
value setf_basic_diffuse_color : material -> color -> material;;
value setf_basic_emissive_color : material -> color -> material;;
value setf_basic_shininess : material -> float -> material;;
value setf_basic_specular_color : material -> color -> material;;
value setf_basic_transparency : material -> float -> material;;
value setf_material_property : vrml_positionable -> (material -> 'a -> material) -> 'a -> vrml_positionable;;
value setf_ambient_intensity : vrml_positionable -> float -> vrml_positionable;;
value setf_diffuse_color : vrml_positionable -> color -> vrml_positionable;;
value setf_emissive_color : vrml_positionable -> color -> vrml_positionable;;
value setf_shininess : vrml_positionable -> float -> vrml_positionable;;
value setf_specular_color : vrml_positionable -> color -> vrml_positionable;;
value setf_transparency : vrml_positionable -> float -> vrml_positionable;;
value new_basic_texture : textureType -> bool -> bool -> vect2D -> float -> vect2D -> vect2D -> texture;;
value new_basic_imageTexture : string list -> textureType;;
value new_basic_movieTexture : float -> clip -> textureType;;
value new_basic_pixelTexture : image -> textureType;;
value new_imageTexture : string list -> texture;;
value new_pixelTexture : image -> texture;;
value new_complete_movieTexture : float -> clip -> texture;;
value new_movieTexture : clip -> texture;;
value setf_basic_texture_repeatS : texture -> bool -> texture;;
value setf_basic_texture_repeatT : texture -> bool -> texture;;
value setf_basic_texture_type : texture -> textureType -> texture;;
value setf_basic_texture_center : texture -> vect2D -> texture;;
value setf_basic_texture_rotation : texture -> float -> texture;;
value setf_basic_texture_scale : texture -> vect2D -> texture;;
value setf_basic_texture_translation : texture -> vect2D -> texture;;
value setf_basic_imageTexture_urls : textureType -> string list -> textureType;;
value setf_basic_pixelTexture_image : textureType -> image -> textureType;;
value setf_basic_movieTexture_speed : textureType -> float -> textureType;;
value setf_basic_movieTexture_clip : textureType -> clip -> textureType;;
value setf_texture_property : vrml_positionable -> (texture -> 'a -> texture) -> 'a -> vrml_positionable;;
value setf_texture_repeatS : vrml_positionable -> bool -> vrml_positionable;;
value setf_texture_repeatT : vrml_positionable -> bool -> vrml_positionable;;
value setf_texture_type : vrml_positionable -> textureType -> vrml_positionable;;
value setf_texture_center : vrml_positionable -> vect2D -> vrml_positionable;;
value setf_texture_rotation : vrml_positionable -> float -> vrml_positionable;;
value setf_texture_scale : vrml_positionable -> vect2D -> vrml_positionable;;
value setf_texture_translation : vrml_positionable -> vect2D -> vrml_positionable;;
value setf_imageTexture_urls : vrml_positionable -> string list -> vrml_positionable;;
value setf_pixelTexture_image : vrml_positionable -> image -> vrml_positionable;;
value setf_movieTexture_speed : vrml_positionable -> float -> vrml_positionable;;
value setf_movieTexture_clip : vrml_positionable -> clip -> vrml_positionable;;

(* map.ml *)
value mapp : 'a list -> ('a -> 'b -> 'c) -> 'b -> 'c list;;
value map_obj : vrml_object -> (vrml_positionable -> 'a -> vrml_positionable) -> 'a -> vrml_object;;
value map_all : vrml_object -> (vrml_positionable -> 'a -> vrml_positionable) -> 'a -> vrml_object;;
value map_imageTexture_url : vrml_object -> string -> vrml_object;;
value map_texture : vrml_object -> string -> vrml_object;;
value map_pixelTexture_image : vrml_object -> image -> vrml_object;;
value map_movieTexture_speed : vrml_object -> float -> vrml_object;;
value map_movieTexture_clip : vrml_object -> clip -> vrml_object;;
value map_diffuse_color : vrml_object -> color -> vrml_object;;
value map_color : vrml_object -> color -> vrml_object;;
value map_all_color : vrml_object -> color -> vrml_object;;
value map_all_imageTexture_urls : vrml_object -> string -> vrml_object;;
value map_all_texture : vrml_object -> string -> vrml_object;;
value map_all_pixelTexture_image : vrml_object -> image -> vrml_object;;
value map_all_movieTexture_clip : vrml_object -> clip -> vrml_object;;
value map_all_movieTexture_speed : vrml_object -> float -> vrml_object;;

(* complex_geometric.ml *)
value new_basic_elevationGrid : color list -> textureCoordinate -> float list -> bool -> bool -> bool -> int -> float -> int -> float -> shape;;
value new_complete_elevationGrid : color list -> textureCoordinate -> float list -> bool -> bool -> bool -> int -> float -> int -> float -> vrml_object;;
value new_elevationGrid : float list -> int -> int -> float -> float -> vrml_object;;
value setf_basic_grid_color : shape -> color list -> shape;;
value setf_basic_grid_texture_coordinate : shape -> textureCoordinate -> shape;;
value setf_basic_grid_height : shape -> float list -> shape;;
value setf_basic_grid_ccw : shape -> bool -> shape;;
value setf_basic_grid_color_per_vertex : shape -> bool -> shape;;
value setf_basic_grid_solid : shape -> bool -> shape;;
value setf_basic_grid_x_dimension : shape -> int -> shape;;
value setf_basic_grid_x_spacing : shape -> float -> shape;;
value setf_basic_grid_z_dimension : shape -> int -> shape;;
value setf_basic_grid_z_spacing : shape -> float -> shape;;
value setf_elevationGrid_colors : vrml_positionable -> color list -> vrml_positionable;;
value setf_elevationGrid_texture_coordinate : vrml_positionable -> textureCoordinate -> vrml_positionable;;
value setf_elevationGrid_height : vrml_positionable -> float list -> vrml_positionable;;
value setf_elevationGrid_ccw : vrml_positionable -> bool -> vrml_positionable;;
value setf_elevationGrid_color_per_vertex : vrml_positionable -> bool -> vrml_positionable;;
value setf_elevationGrid_solid : vrml_positionable -> bool -> vrml_positionable;;
value setf_elevationGrid_x_dimension : vrml_positionable -> int -> vrml_positionable;;
value setf_elevationGrid_x_spacing : vrml_positionable -> float -> vrml_positionable;;
value setf_elevationGrid_z_dimension : vrml_positionable -> int -> vrml_positionable;;
value setf_elevationGrid_z_spacing : vrml_positionable -> float -> vrml_positionable;;
value new_basic_extrusion : bool -> bool -> bool -> vect2D list -> bool -> rotation list -> vect2D list -> bool -> vect3D list -> shape;;
value new_complete_extrusion : bool -> bool -> bool -> vect2D list -> bool -> rotation list -> vect2D list -> bool -> vect3D list -> vrml_object;;
value new_extrusion : vect2D list -> vect3D list -> rotation list -> vrml_object;;
value new_extrusion_with_scales : vect2D list -> vect3D list -> rotation list -> vect2D list -> vrml_object;;
value setf_basic_extrusion_begin_cap : shape -> bool -> shape;;
value setf_basic_extrusion_ccw : shape -> bool -> shape;;
value setf_basic_extrusion_convex : shape -> bool -> shape;;
value setf_basic_extrusion_cross_section : shape -> vect2D list -> shape;;
value setf_basic_extrusion_end_cap : shape -> bool -> shape;;
value setf_basic_extrusion_orientation : shape -> rotation list -> shape;;
value setf_basic_extrusion_scale : shape -> vect2D list -> shape;;
value setf_basic_extrusion_solid : shape -> bool -> shape;;
value setf_basic_extrusion_spine : shape -> vect3D list -> shape;;
value setf_extrusion_begin_cap : vrml_positionable -> bool -> vrml_positionable;;
value setf_extrusion_ccw : vrml_positionable -> bool -> vrml_positionable;;
value setf_extrusion_convex : vrml_positionable -> bool -> vrml_positionable;;
value setf_extrusion_cross_section : vrml_positionable -> vect2D list -> vrml_positionable;;
value setf_extrusion_end_cap : vrml_positionable -> bool -> vrml_positionable;;
value setf_extrusion_orientation : vrml_positionable -> rotation list -> vrml_positionable;;
value setf_extrusion_scale : vrml_positionable -> vect2D list -> vrml_positionable;;
value setf_extrusion_solid : vrml_positionable -> bool -> vrml_positionable;;
value setf_extrusion_spine : vrml_positionable -> vect3D list -> vrml_positionable;;
value new_basic_faceSet : face list -> textureCoordinate -> bool -> bool -> bool -> int list -> shape;;
value new_complete_faceSet : face list -> textureCoordinate -> bool -> bool -> bool -> int list -> vrml_object;;
value new_faceSet : face list -> vrml_object;;
value setf_basic_faceSet_faces : shape -> face list -> shape;;
value setf_basic_faceSet_texture_coordinate : shape -> textureCoordinate -> shape;;
value setf_basic_faceSet_ccw : shape -> bool -> shape;;
value setf_basic_faceSet_convex : shape -> bool -> shape;;
value setf_basic_faceSet_solid : shape -> bool -> shape;;
value setf_basic_faceSet_texture_coordinate_index : shape -> int list -> shape;;
value setf_faceSet_faces : vrml_positionable -> face list -> vrml_positionable;;
value setf_faceSet_texture_coordinate : vrml_positionable -> textureCoordinate -> vrml_positionable;;
value setf_faceSet_ccw : vrml_positionable -> bool -> vrml_positionable;;
value setf_faceSet_convex : vrml_positionable -> bool -> vrml_positionable;;
value setf_faceSet_solid : vrml_positionable -> bool -> vrml_positionable;;
value setf_faceSet_texture_coordinate_index : vrml_positionable -> int list -> vrml_positionable;;

(* map_complex.ml *)
value map_elevationGrid_colors : vrml_object -> color list -> vrml_object;;
value map_elevationGrid_texture_coordinates : vrml_object -> textureCoordinate -> vrml_object;;

(* lights.ml *)
value new_basic_directionalLight : vect3D -> lightType;;
value setf_basic_directionalLight_direction : lightType -> vect3D -> lightType;;
value new_basic_pointLight : vect3D -> float -> lightType;;
value setf_basic_pointLight_attenuation : lightType -> vect3D -> lightType;;
value setf_basic_pointLight_radius : lightType -> float -> lightType;;
value new_basic_spotLight : vect3D -> float -> float -> vect3D -> float -> lightType;;
value setf_basic_spotLight_attenuation : lightType -> vect3D -> lightType;;
value setf_basic_spotLight_beam_width : lightType -> float -> lightType;;
value setf_basic_spotLight_cut_off_angle : lightType -> float -> lightType;;
value setf_basic_spotLight_direction : lightType -> vect3D -> lightType;;
value setf_basic_spotLight_radius : lightType -> float -> lightType;;
value new_basic_light : lightType -> color -> float -> float -> vrml_positionable;;
value setf_light_color : vrml_positionable -> color -> vrml_positionable;;
value setf_light_ambient_intensity : vrml_positionable -> float -> vrml_positionable;;
value setf_light_intensity : vrml_positionable -> float -> vrml_positionable;;
value setf_light_type : vrml_positionable -> lightType -> vrml_positionable;;
value setf_light_type_property : vrml_positionable -> (lightType -> 'a -> lightType) -> 'a -> vrml_positionable;;
value setf_directionalLight_direction : vrml_positionable -> vect3D -> vrml_positionable;;
value setf_pointLight_attenuation : vrml_positionable -> vect3D -> vrml_positionable;;
value setf_pointLight_radius : vrml_positionable -> float -> vrml_positionable;;
value setf_spotLight_attenuation : vrml_positionable -> vect3D -> vrml_positionable;;
value setf_spotLight_beam_width : vrml_positionable -> float -> vrml_positionable;;
value setf_spotLight_cut_off_angle : vrml_positionable -> float -> vrml_positionable;;
value setf_spotLight_direction : vrml_positionable -> vect3D -> vrml_positionable;;
value setf_spotLight_radius : vrml_positionable -> float -> vrml_positionable;;
value new_complete_directionalLight : color -> float -> float -> vect3D -> vrml_object;;
value new_directionalLight : color -> float -> vect3D -> vrml_object;;
value new_complete_pointLight : color -> float -> float -> vect3D -> float -> vrml_object;;
value new_pointLight : color -> float -> float -> vrml_object;;
value new_complete_spotLight : color -> float -> float -> vect3D -> float -> float -> vect3D -> float -> vrml_object;;
value new_spotLight : color -> float -> vect3D -> float -> vrml_object;;

(* map_light.ml *)
value map_light_color : vrml_object -> color -> vrml_object;;
value map_directionalLight_direction : vrml_object -> vect3D -> vrml_object;;

(* sound.ml *)
value new_basic_sound : vect3D -> float -> float -> float -> float -> float -> float -> bool -> string -> float -> clip -> vrml_positionable;;
value new_complete_sound : vect3D -> float -> float -> float -> float -> float -> float -> bool -> string -> float -> clip -> vrml_object;;
value new_sound : vect3D -> float -> clip -> vrml_object;;
value setf_sound_direction : vrml_positionable -> vect3D -> vrml_positionable;;
value setf_sound_intensity : vrml_positionable -> float -> vrml_positionable;;
value setf_sound_max_back : vrml_positionable -> float -> vrml_positionable;;
value setf_sound_max_front : vrml_positionable -> float -> vrml_positionable;;
value setf_sound_min_back : vrml_positionable -> float -> vrml_positionable;;
value setf_sound_min_front : vrml_positionable -> float -> vrml_positionable;;
value setf_sound_priority : vrml_positionable -> float -> vrml_positionable;;
value setf_sound_spatialize : vrml_positionable -> bool -> vrml_positionable;;
value setf_sound_description : vrml_positionable -> string -> vrml_positionable;;
value setf_sound_pitch : vrml_positionable -> float -> vrml_positionable;;
value setf_sound_clip : vrml_positionable -> clip -> vrml_positionable;;

(* frame_manipulations.ml *)
value get_x1 : frame -> float;;
value get_x2 : frame -> float;;
value get_y1 : frame -> float;;
value get_y2 : frame -> float;;
value get_z1 : frame -> float;;
value get_z2 : frame -> float;;
value place_top : vrml_object -> vrml_object -> vrml_object;;
value place_bottom : vrml_object -> vrml_object -> vrml_object;;
value place_right : vrml_object -> vrml_object -> vrml_object;;
value place_left : vrml_object -> vrml_object -> vrml_object;;
value place_front : vrml_object -> vrml_object -> vrml_object;;
value place_behind : vrml_object -> vrml_object -> vrml_object;;
value stick_top : vrml_object -> vrml_object -> vrml_object;;
value stick_bottom : vrml_object -> vrml_object -> vrml_object;;
value stick_right : vrml_object -> vrml_object -> vrml_object;;
value stick_left : vrml_object -> vrml_object -> vrml_object;;
value stick_front : vrml_object -> vrml_object -> vrml_object;;
value stick_behind : vrml_object -> vrml_object -> vrml_object;;

(* special.ml *)
value new_basic_background : float list -> color list -> string list -> string list -> string list -> string list -> string list -> string list -> float list -> color list -> vrml_special;;
value new_complete_background : float list -> color list -> string list -> string list -> string list -> string list -> string list -> string list -> float list -> color list -> vrml_special;;
value new_background : string -> color -> vrml_special;;
value setf_background_ground_angles : vrml_special -> float list -> vrml_special;;
value setf_background_ground_colors : vrml_special -> color list -> vrml_special;;
value setf_background_back_urls : vrml_special -> string list -> vrml_special;;
value setf_background_bottom_urls : vrml_special -> string list -> vrml_special;;
value setf_background_front_urls : vrml_special -> string list -> vrml_special;;
value setf_background_left_urls : vrml_special -> string list -> vrml_special;;
value setf_background_right_urls : vrml_special -> string list -> vrml_special;;
value setf_background_top_urls : vrml_special -> string list -> vrml_special;;
value setf_background_sky_angles : vrml_special -> float list -> vrml_special;;
value setf_background_sky_colors : vrml_special -> color list -> vrml_special;;
value new_basic_fog : color -> string -> float -> vrml_special;;
value linear_fog : string;;
value exponential_fog : string;;
value new_complete_fog : color -> string -> float -> vrml_special;;
value new_linear_fog : color -> vrml_special;;
value new_exponential_fog : color -> vrml_special;;
value setf_fog_color : vrml_special -> color -> vrml_special;;
value setf_fog_type : vrml_special -> string -> vrml_special;;
value setf_fog_visibility_range : vrml_special -> float -> vrml_special;;

(* world.ml *)
value new_vrml_world : string list -> string -> vrml_object -> vrml_special list -> animation list -> vrml_world;;
value set_info : vrml_world -> string list -> vrml_world;;
value set_title : vrml_world -> string -> vrml_world;;
value set_tree : vrml_world -> vrml_object -> vrml_world;;
value set_specials : vrml_world -> vrml_special list -> vrml_world;;
value add_special : vrml_world -> vrml_special -> vrml_world;;
value set_animations : vrml_world -> animation list -> vrml_world;;
value add_animation : vrml_world -> animation -> vrml_world;;

(* translate.ml *)
type dictionary;;

(*
type dictionary = 
  { name : string
  ; triggers : trigger list
  }
;;
*)
type table;;
(*
type table = 
  { trigger_table : (trigger * string) list
  ; object_table : (vrml_object * dictionary) list
  }
;;
*)
value dico : table ref;;
value object_equivalent : vrml_object -> vrml_object -> bool;;
value object_table_find : vrml_object -> dictionary;;
value object_table_remove : vrml_object -> unit;;
value object_table_add : vrml_object -> dictionary -> unit;;
value dico_clear : unit -> unit;;
value trigger_equivalent : trigger -> trigger -> bool;;
value trigger_name : trigger -> string;;
value new_trigger_name : trigger -> string -> unit;;
value new_trigger : vrml_object -> trigger -> unit;;
value new_object_name : vrml_object -> string -> unit;;
value object_table_construct : vrml_world -> unit;;
value reset : unit -> unit;;
value gensym : unit -> string;;
value spaced_float : float -> string;;
value string_of_scalar : float -> string;;
value quote : string -> string;;
value string_of_vect2D : float * float -> string;;
value vect2D_translate : float * float -> string;;
value string_of_vect3D : float * float * float -> string;;
value vect3D_translate : float * float * float -> string;;
value bool_translate : bool -> string;;
value concapply : ('a -> string) -> 'a list -> string;;
value rotation_translate : (float * float * float) * float -> string;;
value translate_list : ('a -> string) -> 'a list -> string;;
value rotation_list_translate : ((float * float * float) * float) list -> string;;
value vect3D_list_translate : (float * float * float) list -> string;;
value point_color_list_translate : point list -> string;;
value index_translate : int list -> string;;
value scalar_list_translate : float list -> string;;
value point_coordinate_list_translate : point list -> string;;
value vect2D_list_translate : (float * float) list -> string;;
value string_list_translate : string list -> string;;
value url_translate : string list -> string;;
value compare : 'a -> 'a -> ('a -> string) -> string;;
value box_translate : float * float * float -> float * float * float -> string;;
value material_translate : material -> string;;
value common_texture_translate : textureItem -> string;;
value hexa : int -> string;;
value clip_translate : clip -> string;;
value image_translate : image -> string;;
value color_translate : point list -> string;;
value coordinate_translate : point list -> string;;
value texCoord_translate : (float * float) list -> string;;
value pointSet_translate : point list -> string;;
value close_index : int list -> int list;;
value create_index : 'a -> 'a list -> int list -> 'a list * int list;;
value multi_create_index : 'a list -> 'a list -> int list -> 'a list * int list;;
value lineSet_translate : line list -> string;;
value unlitGeometric_translate : unlitGeometric -> string;;
value textureCoordinate_translate : (float * float) list -> string;;
value create_face_list_index : 'a list list -> 'a list * int list;;
value faceSet_translate : faceSet -> string;;
value elevationGrid_translate : elevationGrid -> string;;
value extrusion_translate : extrusion -> string;;
value pixelTexture_translate : textureItem -> string;;
value movieTexture_translate : textureItem -> string;;
value imageTexture_translate : textureItem -> string;;
value texture_translate : texture -> string;;
value font_style_translate : fontStyle -> string;;
value recenter_translation : shape -> float * float * float;;
value geometric_translate : shape -> string;;
value shape_translate : geometric -> string;;
value sound_translate : sound -> string;;
value light_translate : light -> string;;
value viewPoint_translate : viewPoint -> string;;
value proximityBox_translate : proximityBox -> string;;
value is_a_real_transform : vrml_object -> bool;;
value compute_proximity_name : string -> trigger -> trigger list -> string;;
value sensor_translate : string -> 'a -> vrml_object -> string;;
value collisionBox_translate : collisionBox -> string;;
value billboard_translate : string -> 'a -> vrml_object -> string;;
value collision_translate : string -> 'a -> vrml_object -> string;;
value transform_translate : string -> 'a -> vrml_object -> string;;
value vrml_object_translate : 'a -> vrml_object -> string;;
value anchor_translate : string -> 'a -> vrml_object -> string;;
value vrml_positionnable_translate : string -> vrml_object -> 'a -> vrml_positionable -> string;;
value lod_translate : 'a -> lod -> string;;
value real_vrml_object_translate : 'a -> vrml_object -> string;;
value timeSensor_translate : string -> float -> bool -> bool -> float -> float -> string;;
value instant_script : string;;
value memory_script : string;;
value script_translate : string -> string -> string;;
value new_route : 'a list ref -> 'a -> unit;;
value new_routes : 'a list ref -> 'a list -> unit;;
(*
value instant_route_translate : 'a -> string -> ('b, string) t -> 'b -> 'a * string;; 
value start_instant_route : string -> ('a, string) t -> 'a -> string * string;;
value stop_instant_route : string -> ('a, string) t -> 'a -> string * string;;
*)
value trigger_create : 'a -> (string * string) list ref -> trigger -> string;;
value start_trigger_route_translate : 'a -> 'b -> string -> 'c -> trigger -> (string * string) list;;
value stop_trigger_route_translate : 'a -> 'b -> string -> animation -> trigger -> (string * string) list;;
value start_trigger_translate : 'a -> (string * string) list ref -> string -> 'b -> trigger -> string;;
value stop_trigger_translate : 'a -> (string * string) list ref -> string -> animation -> trigger -> string;;
value interpolator_translate : string -> string -> float list * 'a -> int -> ('a -> string) -> string;;
value interpolator_route_translate : string -> string -> string -> string -> int -> (string * string) list;;
value changing_route_translate : changing -> string -> string -> string -> int -> (string * string) list;;
value changing_list_route_translate : changing list -> string -> string -> string -> int -> (string * string) list;;
value interpolator_translate : string -> string -> float list * 'a -> int -> ('a -> string) -> string;;
value changing_translate : changing -> string -> int -> string;;
value changing_list_translate : changing list -> string -> int -> string;;
value animation_translate : 'a -> (string * string) list ref -> animation -> string;;
value routes_translate : (string * string) list ref -> string;;
value moving_object_translate : animation list -> string;;
value background_translate : background -> string;;
value fog_translate : fog -> string;;
value vrml_special_translate : vrml_special -> string;;
value vrml_special_list_translate : vrml_special list -> string;;
value vrml_world_translate : vrml_world -> string;;
value test : vrml_world -> unit;;
value vrml_translate_to : vrml_world -> string -> unit;;

(* animations.ml *)
value new_animation : vrml_object -> trigger list -> trigger list -> bool -> float -> bool -> changing list -> animation;;
value setf_animation_target : animation -> vrml_object -> animation;;
value setf_animation_start_trigger : animation -> trigger list -> animation;;
value setf_animation_stop_trigger : animation -> trigger list -> animation;;
value setf_animation_loop : animation -> bool -> animation;;
value setf_animation_duration : animation -> float -> animation;;
value setf_animation_memory : animation -> bool -> animation;;
value setf_animation_movie : animation -> changing list -> animation;;

(* traject.ml *)
value initialize_object : animation -> animation;;
value compute_trajectory : (float -> 'a) -> int -> float list * 'a list;;
value circle_y : float -> float -> float * float * float;;
value circle_z : float -> float -> float * float * float;;
value ellipse : float -> float -> float -> float * float * float;;
value new_extrusion_bis : vect2D list -> vect3D list -> rotation list -> vect2D list -> vrml_object;;
value revolution_orientation : int -> int -> float -> (float * float * float) * float;;
value compute_values : int -> (float -> 'a) -> 'a list;;
value compute_2DShape_y : int -> (float -> 'a * 'b * 'c) -> ('a * 'c) list;;
value identity : 'a -> float * float * float;;
value compute_revolution_extrusion : vect2D list -> float -> int -> vrml_object;;
value couronne : vrml_object;;
value couronne : vrml_object;;
value mon_world : vrml_world;;
