










#open "VRcaML";;
#open "default";;
#open "exceptions";;
#open "basic_types";;
#open "font";;
#open "fonts";;
#open "colors";;
#open "vrml_objects_pre";;
#open "calc_3D";;
#open "vrml_objects_post";;
#open "positionables";;
#open "geometrics";;
#open "geometric_properties";;

(* Map a function with a value on all positionables of a vrml_object *)

(* mapp : 'a list -> ('a -> 'b -> 'c) -> 'b -> 'c list *)
let mapp positionables f valx =
    let rec rmap l =
        match l with
          [] -> []
        | p::tail -> 
             try ((f p valx)::(rmap tail)) with
               Failure _ -> rmap tail
    in
       rmap positionables
;;

(* map_obj :
   vrml_object -> (vrml_positionable -> 'a -> vrml_positionable) -> 'a
-> vrml_object *)
let map_obj object f valx =
    new_vrml_object
       (mapp object.objects f valx) object.children
       object.position object.rotation object.scale
       object.frame object.unique object.anchor
       object.billboard object.collision 
;;

(* map_all :
   vrml_object -> (vrml_positionable -> 'a -> vrml_positionable) -> 'a
-> vrml_object *)
let rec map_all object f valx =
    let rec rmap l =
        match l with
          [] -> []

        | o::tail -> (map_all o f valx)::(rmap tail)
    in
       new_vrml_object
          (mapp object.objects f valx) (rmap object.children)
          object.position object.rotation object.scale
          object.frame object.unique object.anchor
          object.billboard object.collision 
;;

(* map_texture : vrml_object -> string -> vrml_object *)
let map_imageTexture_url object url =
    map_obj object setf_imageTexture_urls [ url ]
;;

let map_texture = map_imageTexture_url ;;

(* map_pixelTexture_image : vrml_object -> image -> vrml_object *)
let map_pixelTexture_image object image =
    map_obj object setf_pixelTexture_image image
;;

(* map_movieTexture_speed : vrml_object -> float -> vrml_object *)
let map_movieTexture_speed object speed =
    map_obj object setf_movieTexture_speed speed
;;

(* map_movieTexture_clip : vrml_object -> clip -> vrml_object *)
let map_movieTexture_clip object clip =
    map_obj object setf_movieTexture_clip clip
;;

(* map_diffuse_color : vrml_object -> color -> vrml_object *)
let map_diffuse_color object color =
    map_obj object setf_diffuse_color color
;;

let map_color = map_diffuse_color ;;

let map_all_color object = map_all object setf_diffuse_color ;;

let map_all_imageTexture_urls object url =
    map_all object setf_imageTexture_urls [ url ]
;;

let map_all_texture = map_all_imageTexture_urls ;;

let map_all_pixelTexture_image object =
    map_all object setf_pixelTexture_image
;;

let map_all_movieTexture_clip object =
    map_all object setf_movieTexture_clip
;;

let map_all_movieTexture_speed object =
    map_all object setf_movieTexture_speed 
;;
